#!/usr/bin/env ruby

require 'qte'
require 'qpe'
require 'thread'
require "date"
require "uri"
require "httpsnapshot.rb"
require "rexml/document"
require "open-uri"
require "timeout"
require "qtexml"
require 'uconv'
require 'parsedate'
require "getopts"

include Qte
include Qpe
include Qtexml

$APPLICATION_NAME = "zRSSReader"

class BerryBaseWidget < QWidget
	def initialize(parent)
			super(parent)
	end

	def close()
		return true
	end

	def showing()
	end

	def repaintFont(font)
	end

	def getWidgetName()
		return ""
	end
end

class Berry < QWidget
	WIDGET_COUNT = 2
	RSS_LIST_WIDGET_ID = 0
	RSS_DESC_VIEW_WIDGET_ID = 1

	def initialize()
		super()
		catchEvent()
		
		setCaption($APPLICATION_NAME)

		
		@widgetStack = QWidgetStack.new(self)
		@rssListWidget = BerryRSSListWidget.new(self)
		@rssDescViewWidget = BerryRSSDescViewWidget.new(self)
		@widgetStack.addWidget(@rssListWidget, RSS_LIST_WIDGET_ID)
		@widgetStack.addWidget(@rssDescViewWidget, RSS_DESC_VIEW_WIDGET_ID)
		
		@verticalLayout = QVBoxLayout.new(self)
		@verticalLayout.addWidget(@widgetStack, 1)
		
		@commMode = false
		@recentWidgetId = nil
		
		changeScreen(RSS_LIST_WIDGET_ID)
	end

	def changeScreen(id)
		@recentWidetId = @currentWidgetId
		@currentWidgetId = id
		@widgetStack.raiseWidget(@currentWidgetId)
		getWidget(id).showing()
		if getWidget(id).getWidgetName() != "" then
			setCaption(tr("#{$APPLICATION_NAME} - #{getWidget(id).getWidgetName()}"))
		else
			setCaption(tr("#{$APPLICATION_NAME}"))
		end
	end
	
	def getRecentWidgetId()
		return @recentWidgetId
	end
	
	def getWidget(id)
		if id == RSS_LIST_WIDGET_ID then
			return @rssListWidget
		end
		if id == RSS_DESC_VIEW_WIDGET_ID then
			return @rssDescViewWidget
		end
		return nil
	end
	
	def closeEvent(event)
		if (getWidget(@currentWidgetId).close()) then
			event.accept()
		end
	end
end

class BerryRSSListWidget < BerryBaseWidget
	def initialize(parent)
		super(parent)
		
		@parent = parent
		
		catchEvent()

		$homedir = "/home/zaurus/Settings/"
		$setfile = "/home/zaurus/Settings/zrssreader.xml"
		$fontname = "lcfont"

		if (!FileTest.exist?($setfile))
			$savedir = "/home/zaurus/Documents/application/zreeget/"
			$fontsize = 12
			$fontsize_item = 0
			$maxitemnum = 100
			$proxyon = false
			$proxyurl = "http://proxy.xxx.co.jp:8080/"
			$browser = "netfront3"
			$browser_item = 0
			$w3mbon = false
		else
			loadsetfile	
		end

		@bloglist = QListView.new(self)
		@bloglist.setAllColumnsShowFocus(true)
		titlewidth = QPEApplication.desktop.width - 20 - 30 -90 -49
		@bloglist.addColumn(tr("S"),20)
		@bloglist.addColumn(tr("Title"),titlewidth)
		@bloglist.addColumn(tr("新着"),30)
		@bloglist.addColumn(tr("date"),90)
		@bloglist.setRootIsDecorated(false)
		@bloglist.setMultiSelection(false)
		@bloglist.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@bloglist.setColumnAlignment(2,AlignHCenter)

		@le0 = QLabel.new("",self)
		@pb0 = QPushButton.new(tr("取得開始"),self)
		@pb1 = QPushButton.new(tr("取得中止"),self)
		@pr0 = QProgressBar.new(self)
		@pr0.setCenterIndicator(true)

		@le0.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@pb0.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@pb1.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@pr0.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		
		@Menu1 = QPopupMenu.new()
		@Menu2 = QPopupMenu.new()
		@Menu3 = QPopupMenu.new()
		@Menu4 = QPopupMenu.new()
		@Menu1.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@Menu2.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@Menu3.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@Menu4.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@m11 = @Menu1.insertItem(tr("取得開始"), self, "allgetstart", Key_R)
		@m12 = @Menu1.insertItem(tr("取得中止"), self, "allgetstop", Key_S)
		@m14 = @Menu1.insertItem(tr("キャッシュ取得開始"), self, "cachegetstart")
		@m15 = @Menu1.insertItem(tr("キャッシュ取得中止"), self, "cachegetstop")
		@m11 = @Menu1.insertItem(tr("このRSSを取得開始"), self, "onegetstart", Key_T)
	#	@m12 = @Menu1.insertItem(tr("ダミーネット接続"), self, "dummynet", Key_N)
		@m13 = @Menu1.insertItem(tr("設定"), self, "setting", Key_O)
		@m21 = @Menu2.insertItem(tr("RSS追加"), self, "addrss", Key_A)
		@m22 = @Menu2.insertItem(tr("RSS修正"), self, "editrss", Key_M)
		@m23 = @Menu2.insertItem(tr("RSS削除"), self, "delrss",CTRL+Key_D)
		@m26 = @Menu2.insertItem(tr("RSS Autodiscovery"), self, "rssAd", Key_L)
		@m24 = @Menu2.insertItem(tr("キャッシュ設定ONOFF"), self, "cacherss", Key_C)
		@m24 = @Menu2.insertItem(tr("すべてのキャッシュ設定ONOFF"), self, "cacheallrss")
		@m25 = @Menu2.insertItem(tr("すべてのRSS削除"), self, "delallrss")
		@m31 = @Menu3.insertItem(tr("インポート"), self, "opmlin")
		@m32 = @Menu3.insertItem(tr("エクスポート"), self, "opmlex")
		@m41 = @Menu4.insertItem(tr("Up"), self, "upkey", Key_A)
		@m42 = @Menu4.insertItem(tr("Down"), self, "downkey", Key_Z)
		@menuBar = QPEMenuBar.new(self)
		@menuBar.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@menuBar.insertItem(tr("Menu"), @Menu1)
		@menuBar.insertItem(tr("RSS設定"), @Menu2)
		@menuBar.insertItem(tr("OPML"), @Menu3)
	#	@menuBar.insertItem(tr("InputHelper連携"), @Menu4)
		
		@widgetLayout = QVBoxLayout.new(self)
		@widgetLayout.addWidget(@bloglist, 1)
		@widgetLayout2 = QHBoxLayout.new(@widgetLayout)
		@widgetLayout2.addWidget(@pb0, 1)
		@widgetLayout2.addWidget(@pb1, 1)
		@widgetLayout3 = QHBoxLayout.new(@widgetLayout)
		@widgetLayout3.addWidget(@le0, 3)
		@widgetLayout3.addWidget(@pr0, 1)
		@widgetLayout.setMenuBar(@menuBar)


		@Menu1.setItemEnabled(@m11,true)
		@Menu1.setItemEnabled(@m12,false)
		@Menu1.setItemEnabled(@m14,true)
		@Menu1.setItemEnabled(@m15,false)
		@pb0.setEnabled(true)		
		@pb1.setEnabled(false)


		if (FileTest.exist?($homedir + "zRSSRtop.xml"))
			readtoplist($homedir + "zRSSRtop.xml")
			setsortlist
		else
			fp = File.open($homedir + "zRSSRtop.xml","w")
			fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp.puts("<zRSSReader>")
			fp.close	
		end

		connect(@pb0,QSIGNAL("clicked()"),self,'allgetstart')
		connect(@pb1,QSIGNAL("clicked()"),self,'allgetstop')
		Qte::connect(@bloglist,QSIGNAL("returnPressed(QListViewItem *)"), self, 'listup')
		Qte::connect(@bloglist,QSIGNAL("doubleClicked(QListViewItem *)"), self, 'listup')
	end

	def cachegetstart

		@t = Thread.start do
			Thread.pass

			@Menu1.setItemEnabled(@m14,false)
			@Menu1.setItemEnabled(@m15,true)

			day1 = Time.now

			count = 0
			item = @bloglist.firstChild()
			@le0.setText(tr("サーチ中"))
			@pr0.setProgress(0)	
			while(!item.nil?)
				rec = row(item)
				@toptitle = rec[1]
				savelist = []
				savelist= rec[4].split('/')
				sedlist = savelist[2].gsub(/\./,"") + "_" + savelist[3].gsub(/\./,"") + "_" + savelist[-1].gsub(/\./,"").sub(/\?/,"") 
				savelist = []

				filePath = $savedir + sedlist + ".xml"

			        fp = File.new(filePath, "r")
				        handler = ZcDataParser.new()
				        source = QXmlInputSource.new()
				        source.setData(tr(fp.read))
				        reader = QXmlSimpleReader.new()
				        reader.setContentHandler(handler)
				        reader.parse(source)
			        fp.close()
			        recordList = handler.recordList()
			        if (recordList.nil?)
					return false
			        end

			        for i in 0..(recordList.length() - 1)
					localc = recordList[i][0]
					@title = recordList[i][1]
					y3 = recordList[i][2]
					y4 = recordList[i][3]
					url2 = recordList[i][4]
					url = recordList[i][5]

					if (localc =~ /^C/ ) then
						tempurl = []
						tempurl = url.split('/')
						urle1 = tempurl.pop
	
						bname = savename_c(url)
		
						if (bname =~/^index.html$/) then
							bname = urle1 + ".html"
						end

						url2t = $savedir + sedlist + "/" + bname
					#	url2 = "file:\/\/" + url2t
						url2t = url2.sub(/file:\/\//,"")

						if ( FileTest.exist?(url2t)) then
						else
							count = count + 1
						end
					end
				end

				item = item.nextSibling()
			end

			item = @bloglist.firstChild()
			countnum = 0.0
			while(!item.nil?)
				rec = row(item)
				@toptitle = rec[1]
				savelist = []
				savelist= rec[4].split('/')
				sedlist = savelist[2].gsub(/\./,"") + "_" + savelist[3].gsub(/\./,"") + "_" + savelist[-1].gsub(/\./,"").sub(/\?/,"") 
				savelist = []

				filePath = $savedir + sedlist + ".xml"

			        fp = File.new(filePath, "r")
				        handler = ZcDataParser.new()
				        source = QXmlInputSource.new()
				        source.setData(tr(fp.read))
				        reader = QXmlSimpleReader.new()
				        reader.setContentHandler(handler)
				        reader.parse(source)
			        fp.close()
			        recordList = handler.recordList()
			        if (recordList.nil?)
					return false
			        end

			        for i in 0..(recordList.length() - 1)
					localc = recordList[i][0]
					@title = recordList[i][1]
					y3 = recordList[i][2]
					y4 = recordList[i][3]
					url2 = recordList[i][4]
					url = recordList[i][5]

					if (localc =~ /^C/ ) then
						tempurl = []
						tempurl = url.split('/')
						urle1 = tempurl.pop
	
						bname = savename_c(url)
		
						if (bname =~/^index.html$/) then
							bname = urle1 + ".html"
						end

						url2t = $savedir + sedlist + "/" + bname
					#	url2 = "file:\/\/" + url2t
						url2t = url2.sub(/file:\/\//,"")

						if ( FileTest.exist?(url2t)) then
						else
							countnum = countnum + 1
							if( localc =~ /^CI/ ) then
								@le0.setText(tr( @toptitle + " " + @title + " 取得開始"))
								getblog(url, $savedir + sedlist, 0, " ")
								@le0.setText(tr( @toptitle + " " + @title + " 取得完了"))
							elsif( localc =~ /^C/ ) then
								@le0.setText(tr( @toptitle + " " + @title + " 取得開始"))
								getblog(url, $savedir + sedlist, 0, "n")
								@le0.setText(tr( @toptitle + " " + @title + " 取得完了"))
							end
							item.setText(2,tr("Ｃ"))
							if( localc =~ /^C/ ) then
								if (FileTest.exist?($savedir + sedlist + "/index.html"))
									File.rename($savedir + sedlist + "/index.html",$savedir + sedlist + "/" + bname)	
									if (FileTest.exist?($savedir + sedlist + "/.cache-snap/index.html"))
										File.delete($savedir + sedlist + "/.cache-snap/index.html")
									end	
									if (FileTest.exist?($savedir + sedlist + "/.list-snap"))
										File.delete($savedir + sedlist + "/.list-snap")
									end
								end
							end
							begin
							Dir::open($savedir + sedlist + "/.cache-snap") {|d|
								d.each {|f|
									if ( f =~ /htm/ ) then
										if (FileTest.exist?($savedir + sedlist + "/.cache-snap/" + f))
											File.delete($savedir + sedlist + "/.cache-snap/" + f)
										end
									end
								}
							}
							rescue
							end
							@pr0.setProgress( (countnum/count) * 100.0 )	
						end
					end
	        		end	

				item = item.nextSibling()
			end
	#	rescue

			item = @bloglist.firstChild()
			fp = File.open($homedir + "zRSSRtop.xml","w")
			fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp.puts("<zRSSReader>")
			while(!item.nil?)
				rec = row(item)
				recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])
				fp.puts	(recStr)
				item = item.nextSibling()
			end
			fp.close

			GC.start
			@Menu1.setItemEnabled(@m14,true)
			@Menu1.setItemEnabled(@m15,false)

			day2 = Time.now
			days = (day2 - day1).divmod(24*60*60)
			hours = days[1].divmod(60*60)
			mins = hours[1].divmod(60)

			gettime = "(#{count} #{mins[0].to_i}:#{mins[1].to_i})"

			@le0.setText(tr("キャッシュ取得完了 " +gettime))
			@pr0.reset
		end
	end

	def cachegetstop
		if @t != nil then
			@t.kill()

			item = @bloglist.firstChild()
			fp = File.open($homedir + "zRSSRtop.xml","w")
			fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp.puts("<zRSSReader>")
			while(!item.nil?)
				rec = row(item)
				recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])
				fp.puts	(recStr)
				item = item.nextSibling()
			end
			fp.close
	
			GC.start
			@Menu1.setItemEnabled(@m14,true)
			@Menu1.setItemEnabled(@m15,false)
			@le0.setText(tr("キャッシュ取得中止"))
			@pr0.reset
		end
	end

	def listup(item)
		title  = item.text(1).utf8().to_str()
		list   = item.text(4).utf8().to_str()

		savelist = []
		savelist= list.split('/')
		sedlist = savelist[2].gsub(/\./,"") + "_" + savelist[3].gsub(/\./,"") + "_" + savelist[-1].gsub(/\./,"").sub(/\?/,"") 
		savelist = []

		if(!FileTest.exist?($savedir + sedlist + ".xml"))
			@le0.setText(tr("オープンエラー"))
			return
		else
			@le0.setText(tr(" "))
		end

		@parent.getWidget(Berry::RSS_DESC_VIEW_WIDGET_ID).screenInit(list,title)
		@parent.changeScreen(Berry::RSS_DESC_VIEW_WIDGET_ID)
	end

	def loadsetfile
		if(!FileTest.exist?($setfile))
			return
		end

		fp = File.open($setfile,"r")
		doc = REXML::Document.new(fp)

		REXML::XPath.match(doc, "/config").each do |i|
			$savedir =  i.elements['savedir'].text
			$maxitemnum = i.elements['maxitem'].text.to_i
			$fontsize = i.elements['fontsize'].text.to_i
			$fontsize_item = i.elements['fontsize_item'].text.to_i
			if(i.elements['proxyon'].text =~ /true/) then
				$proxyon = true
			else
				$proxyon = false
			end
			$proxyurl = i.elements['proxyurl'].text
			$browser = i.elements['browser'].text
			$browser_item = i.elements['browser_item'].text.to_i
			begin
			if(i.elements['w3m_bookmark'].text =~ /true/) then
				$w3mbon = true
			else
				$w3mbon = false
			end
			rescue
				$w3mbon = false
			end
		end
		fp.close()
	end

	def readtoplist(filePath)
	        fp = File.new(filePath, "r")
		        handler = ZcDataParser.new()
		        source = QXmlInputSource.new()
		        source.setData(tr(fp.read))
		        reader = QXmlSimpleReader.new()
		        reader.setContentHandler(handler)
		        reader.parse(source)
	        fp.close()
	        recordList = handler.recordList()
	        if (recordList.nil?)
			return false
	        end

	        for i in 0..(recordList.length() - 1)
			y1 = recordList[i][0]
			y2 = recordList[i][1]
			y3 = recordList[i][2]
			y4 = recordList[i][3]
			y5 = recordList[i][4]
			y6 = recordList[i][5]
			QListViewItem.new(@bloglist, tr(y1),tr(y2),tr(y3),tr(y4),tr(y5),tr(y6))
	        end	
	end

	def opmlin
		$opmlfile = " "
		dlg4 = OpmlDialog.new(self)
		res4 = dlg4.exec()

		if(!FileTest.exist?($opmlfile))
			return
		end

	        fp = File.new($opmlfile, "r")
		        handler = ZcOpmlParser.new()
		        source = QXmlInputSource.new()
		        source.setData(tr(fp.read))
		        reader = QXmlSimpleReader.new()
		        reader.setContentHandler(handler)
		        reader.parse(source)
	        fp.close()
	        recordList = handler.recordList()
	        if (recordList.nil?)
			return false
	        end

	        for i in 0..(recordList.length() - 1)
			y1 = "  "
			y2 = recordList[i][0]
			y3 = " "
			y4 = " "
			y5 = recordList[i][1]
			y6 = " "
			if (!y5.nil?) then
				QListViewItem.new(@bloglist, tr(y1),tr(y2),tr(y3),tr(y4),tr(y5),tr(y6))
			end
	        end

		item = @bloglist.firstChild()

		fp = File.open($homedir + "zRSSRtop.xml","w")
		fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
		fp.puts("<zRSSReader>")
		while(!item.nil?)
			rec = row(item)
			recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])
			fp.puts	(recStr)
			item = item.nextSibling()
		end
		fp.close
	end

	def opmlex
		item = @bloglist.firstChild()
		fp = File.open($savedir + "export.xml","w")
		fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
		fp.puts("<opml version=\"1.0\">")
		fp.puts("\t<head>")
		fp.puts("\t\t<title>zRSSReader feeds</title>")
		fp.puts("\t\t<ownerName>zaurus</ownerName>")
		fp.puts("\t</head>")
		fp.puts("\t<body>")
		while(!item.nil?)
			rec = row(item)
			recStr = "\t\t<outline text=\"" + rec[1] + "\" title=\"" + rec[1] + "\" type=\"rss\" xmlUrl=\"" + rec[4] + "\" />"
			fp.puts	(recStr)
			item = item.nextSibling()
		end
		fp.puts("\t</body>")
		fp.puts("</opml>")
		fp.close
	end

	def rssAd
		dlg5 = RssAdDialog.new(self)
		res = dlg5.exec()
		if ((res == QDialog::Accepted) or dlg5.result2()) then
			@bloglist.clear()
			readtoplist($homedir + "zRSSRtop.xml")
			@bloglist.setSorting(3,false)
		end
	end

	def addrss
		dlg2 = BloglistDialog.new(self)
		res = dlg2.exec()
		if ((res == QDialog::Accepted) or dlg2.result2()) then
			@bloglist.clear()
			readtoplist($homedir + "zRSSRtop.xml")
			@bloglist.setSorting(3,false)
		end
	end

	def delrss
		item = @bloglist.selectedItem()
		@bloglist.takeItem(item)
		item = @bloglist.firstChild()

		fp = File.open($homedir + "zRSSRtop.xml","w")
		fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
		fp.puts("<zRSSReader>")
		while(!item.nil?)
			rec = row(item)
			recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])
			fp.puts	(recStr)
			item = item.nextSibling()
		end
		fp.close
	end


	def delallrss
		@bloglist.clear()
		fp = File.open($homedir + "zRSSRtop.xml","w")
		fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
		fp.puts("<zRSSReader>")
		fp.close
	end

	def cacherss
		begin
			item = @bloglist.selectedItem()
			rec = row(item)
			if (rec[0] =~ /CI/ ) then
				item.setText(0,"C ")
			elsif (rec[0] =~ /C/ ) then
				item.setText(0,"  ")
			else 
				item.setText(0,"CI")
			end

			item = @bloglist.firstChild()
			fp = File.open($homedir + "zRSSRtop.xml","w")
			fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp.puts("<zRSSReader>")
			while(!item.nil?)
				rec = row(item)
				recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])
				fp.puts	(recStr)
				item = item.nextSibling()
			end
			fp.close
		rescue
		end
	end


	def cacheallrss
		begin
			item = @bloglist.firstChild()
			rec = row(item)
			if (rec[0] =~ /CI/ ) then
				cachec = "C "
			elsif (rec[0] =~ /C/ ) then
				cachec = "  "
			else 
				cachec = "CI"
			end

			item = @bloglist.firstChild()
			fp = File.open($homedir + "zRSSRtop.xml","w")
			fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp.puts("<zRSSReader>")
			while(!item.nil?)
				item.setText(0,cachec)
				rec = row(item)
				recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])
				fp.puts	(recStr)
				item = item.nextSibling()
			end
			fp.close
		rescue
		end
	end

	def editrss
		item = @bloglist.selectedItem()
		rec = row(item)
		$editrss = rec[4]
		dlg4 = BlogeditDialog.new(self)
		res4 = dlg4.exec()

		if ((res4 == QDialog::Accepted) or dlg4.result2()) then
			begin
			item.setText(4,$editrss)

			item = @bloglist.firstChild()
			fp = File.open($homedir + "zRSSRtop.xml","w")
			fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp.puts("<zRSSReader>")
			while(!item.nil?)
				rec = row(item)
				recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])
				fp.puts	(recStr)
				item = item.nextSibling()
			end
			fp.close
			rescue
			end
		end
	end

	def setting
		dlg3 = BlogSettingDialog.new(self)
		res3 = dlg3.exec()
	end

	def upkey
		system(" qcop QPE/InputHelper 'key(QString)' 'click Up' ")
	end

	def downkey
		system(" qcop QPE/InputHelper 'key(QString)' 'click Down' ")
	end


	def dummynet
		system("qcop QPE/Network 'up()'")
	end

	def row(item)
		rec = Array.new(6)
		rec[0] = item.text(0).utf8().to_str()
		rec[1] = item.text(1).utf8().to_str()
		rec[2] = item.text(2).utf8().to_str()
		rec[3] = item.text(3).utf8().to_str()
		rec[4] = item.text(4).utf8().to_str()
		rec[5] = item.text(5).utf8().to_str()
		return rec
	end


	def setsortlist
		@bloglist.setSorting(3,false)
		item = @bloglist.firstChild()
		@bloglist.setCurrentItem(item)
		@bloglist.setFocus()
	end

	def allgetstop
		if @t2 != nil then
			@t2.kill()
		end
		if @t != nil then
			@t.kill()

			if @allget then
				countget = 2
				@temptop.each do |line|
					countget = countget +1
				end
	
				count = 0
				fp = File.open($homedir + "zRSSRtop.xml","r")
					fp.each do |line|
						count = count + 1
						if( count > countget) then
							@temptop.push line
						end
					end
				fp.close
	
				fp = File.open($homedir + "zRSSRtop.xml","w")
					fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
					fp.puts("<zRSSReader>")
					@temptop.each do |line|
						fp.puts(line)
					end
   				#	fp.puts("</zRSSReader>")
				fp.close
	
	
				@bloglist.clear()
				readtoplist($homedir + "zRSSRtop.xml")
			end

			if $w3mbon then
				savetoplistw3mb
			end
	
			GC.start
			@Menu1.setItemEnabled(@m11,true)
			@Menu1.setItemEnabled(@m12,false)
			@pb0.setEnabled(true)		
			@pb1.setEnabled(false)
			@le0.setText(tr("取得中止"))
			@pr0.reset
		end
	end

	def savestr(str0,str1,str2,str3,str4,str5)
		recStr = "\t<list mode=\""
		recStr += str0
		recStr += "\" title=\""
		recStr += escapehtml(str1).chomp.gsub(/\r/m,"").gsub(/\n/m,"")
		recStr += "\" flg=\""
		recStr += str2
		recStr += "\" date=\""
		recStr += str3
		recStr += "\" url=\""
		recStr += escapehtml(str4)
		recStr += "\" url2=\""
		recStr += escapehtml(str5)
		recStr += "\" />\n"
		return recStr
	end


	def savew3mb(str0,str1,str2,str3,str4,str5)
		recStr = "<li>"
		recStr += str0 + " "
		recStr += "<a href=\""
		recStr += escapehtml(str4)
		recStr += "\">C</a> "
		recStr += "<a href=\""
		recStr += escapehtml(str5)
		recStr += "\">O</a> " + str2 + " " + str3 + " " 
		recStr += escapehtml(str1).chomp.gsub(/\r/m,"").gsub(/\n/m,"") + " \n"
		return recStr
	end

	def savetopw3mb(str0,str1,str2,str3,str4,str5)
		recStr = "<li>"
		recStr += str3 + " " + str0 + " " + str2 + " "
		recStr += "<a href=\""
		recStr += escapehtml(str4)
		recStr += "\">"
		recStr += escapehtml(str1).chomp.gsub(/\r/m,"").gsub(/\n/m,"") + "</a> \n"
		return recStr
	end

	def savetoplistw3mb
		savelist = []
		rec = []
		item = @bloglist.firstChild()
		fp = File.open($savedir + "zRSSReader_w3m.html","w")
	   	fp.puts("<html><head><title>zRSSReader Bookmarks</title></head>")
		fp.puts("<body>")
		fp.puts("<ul>")
		while(!item.nil?)
			rec = row(item)
			savelist = rec[4].split('/')
			begin
				blog = savelist[2].gsub(/\./,"") + "_" + savelist[3].gsub(/\./,"") + "_" + savelist[-1].gsub(/\./,"").sub(/\?/,"") 
				w3mbkname = $savedir + "w3m" + blog + ".html"
				recStr = savetopw3mb(rec[0],rec[1],rec[2],rec[3],w3mbkname,rec[5])
				fp.puts	(recStr)
			rescue
			end
			item = item.nextSibling()
		end
		fp.puts("</ul>")
		fp.close

		savelist = []
		rec = []
	end

	def escapehtml(str)
		str = str.gsub(/\&/,"\&amp\;").gsub(/\"/,"\&quot\;").gsub(/</,"\&lt\;").gsub(/>/,"\&gt\;").gsub(/'/,"\&apos\;")
		return str
	end

	def allgetstart
		@t = Thread.start do
			Thread.pass

			@allget = true
			@Menu1.setItemEnabled(@m11,false)
			@Menu1.setItemEnabled(@m12,true)
			@pb0.setEnabled(false)		
			@pb1.setEnabled(true)

			@le0.setText(tr("取得開始"))
			day1 = Time.now
			savelist = []
			@temptop = []
			list = []

			if(!FileTest.exist?($savedir))
				Dir.mkdir($savedir)
			end

			@pr0.setProgress(0)
			count = @bloglist.childCount.to_i
			@step = 100.00 / count


		        fp = File.new($homedir + "zRSSRtop.xml", "r")
			        handler = ZcDataParser.new()
			        source = QXmlInputSource.new()
			        source.setData(tr(fp.read))
			        reader = QXmlSimpleReader.new()
			        reader.setContentHandler(handler)
			        reader.parse(source)
		        fp.close()
		        recordList = handler.recordList()
		        if (recordList.nil?)
				return false
		        end

		        for i in 0..(recordList.length() - 1)
				y1 = recordList[i][0]
				y2 = recordList[i][1]
				y3 = recordList[i][2]
				y4 = recordList[i][3]
				y5 = recordList[i][4]
				y6 = recordList[i][5]
				savelist = y5.split('/')
				blog = savelist[2].gsub(/\./,"") + "_" + savelist[3].gsub(/\./,"") + "_" + savelist[-1].gsub(/\./,"").sub(/\?/,"") 
				@toptitle = y2
				list = recordList[i]
				makebloglist(y5,blog,1,y1,@step * (i+1),list)

				@pr0.setProgress(@step * (i+1))
		        end	

	
			fp = File.open($homedir + "zRSSRtop.xml","w")
				fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
				fp.puts("<zRSSReader>")
				@temptop.each do |line|
					fp.puts(line)
				end
   			#	fp.puts("</zRSSReader>")
			fp.close
	
			@pr0.setProgress(100)

			@bloglist.clear()
			readtoplist($homedir + "zRSSRtop.xml")
			day2 = Time.now

			days = (day2 - day1).divmod(24*60*60)
			hours = days[1].divmod(60*60)
			mins = hours[1].divmod(60)

			gettime = "(#{count} #{mins[0].to_i}:#{mins[1].to_i})"
			
			if $w3mbon then
				savetoplistw3mb
			end

			if @errorflag then
				@le0.setText(tr("取得完了（一部取得できなかったサイトあり） " + gettime))
			else
				@le0.setText(tr("取得完了 " + gettime))
			end

			@Menu1.setItemEnabled(@m11,true)
			@Menu1.setItemEnabled(@m12,false)
			@pb0.setEnabled(true)		
			@pb1.setEnabled(false)

			savelist = []
			@temptop = []
			list = []

			GC.start
			@pr0.reset
			@allget = false
		end
	end

	def onegetstart
		@allget = false
		@t = Thread.start do
			Thread.pass

			@Menu1.setItemEnabled(@m11,false)
			@Menu1.setItemEnabled(@m12,true)
			@pb0.setEnabled(false)		
			@pb1.setEnabled(true)

			if(!FileTest.exist?($savedir))
				Dir.mkdir($savedir)
			end

			@le0.setText(tr("取得開始"))
			day1 = Time.now
			savelist = []
			@temptop = []
			list = []

			@pr0.setProgress(0)


			item = @bloglist.selectedItem()
			rec = row(item)

			savelist = rec[4].split('/')
			blog = savelist[2].gsub(/\./,"") + "_" + savelist[3].gsub(/\./,"") + "_" + savelist[-1].gsub(/\./,"").sub(/\?/,"") 
			list.push rec[0]
			list.push rec[1]
			list.push rec[2]
			list.push rec[3]
			list.push rec[4]
			list.push rec[5]

			@toptitle = rec[1]
			@step = 100
			makebloglist(rec[4],blog,1,rec[0],100,list)

			item.setText(1,tr(@toptitle))
			item.setText(2,tr(@status))
			item.setText(3,tr(@newdate))


			begin	
				item = @bloglist.firstChild()
				fp = File.open($homedir + "zRSSRtop.xml","w")

				fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
				fp.puts("<zRSSReader>")
				while(!item.nil?)
					rec = row(item)
					recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])

					fp.puts	(recStr)
					item = item.nextSibling()
				end
   				#fp.puts("</zRSSReader>")
				fp.close
			rescue
			end
			
			@pr0.setProgress(100)

			day2 = Time.now

			days = (day2 - day1).divmod(24*60*60)
			hours = days[1].divmod(60*60)
			mins = hours[1].divmod(60)

			gettime = "(1 #{mins[0].to_i}:#{mins[1].to_i})"

			if $w3mbon then
				savetoplistw3mb
			end

			if @errorflag then
				@le0.setText(tr("取得エラー " + gettime))
			else
				@le0.setText(tr("取得完了 " + gettime))
			end

			@Menu1.setItemEnabled(@m11,true)
			@Menu1.setItemEnabled(@m12,false)
			@pb0.setEnabled(true)		
			@pb1.setEnabled(false)


			savelist = []
			@temptop = []
			list = []

			GC.start
			@pr0.reset
		end
	end

        def makebloglist(rssurl,dir,linkno,localc,prnum,linelist)
		templist = []
		savelist = []
		tempurl = []
		tempdate = []
		datenum = []
		w3mlist = []
		timei = 0
		@status = "　"
		@title = " "
		itemcount = 1

		@pr0.setProgress(prnum - @step * 0.9 )
		@le0.setText(tr(@toptitle + " RSS取得中"))
		getblog(rssurl,$savedir + dir,0,"gUn")

		rssnamet =  rssurl.split('/')
		rssname = rssnamet[-1]

		if(rssname =~ /^\?/) != nil then
			rssname = "index.rdf"
			if (FileTest.exist?($savedir + dir + "/index.html")) then
				File.rename($savedir + dir + "/index.html",$savedir + dir + "/" + rssname)	
			elsif (FileTest.exist?($savedir + dir + "/.cache-snap/index.html")) then
				File.rename($savedir + dir + "/.cache-snap/index.html",$savedir + dir + "/" + rssname)	
			end
			if (FileTest.exist?($savedir + dir + "/.list-snap"))
				File.delete($savedir + dir + "/.list-snap")
			end
		else
			if (rssname =~ /\?/) != nil then
				rssnamet = rssname.split('?')
				rssname = rssnamet[0]
			end
		end

		@le0.setText(tr(@toptitle + " RSS取得完了"))
		if $DEBUG then
			print rssname
		end

		if (FileTest.exist?($savedir + dir + "/.cache-snap/" + rssname + ".html"))
			File.rename($savedir + dir + "/.cache-snap/" + rssname + ".html",$savedir + dir + "/" + rssname)	
		end

		if (FileTest.exist?($savedir + dir + "/.cache-snap/" + rssname))
			File.rename($savedir + dir + "/.cache-snap/" + rssname,$savedir + dir + "/" + rssname)	
		end

		if (!FileTest.exist?(($savedir + dir + "/" + rssname))) then
			if (!FileTest.exist?(($savedir + dir + "/index.html"))) then
				@status = "×"
				@newdate = linelist[3] 
				@toptitle = linelist[1]
	
				recStr = savestr(linelist[0],linelist[1],@status,linelist[3],linelist[4]," ")
				@temptop.push recStr	
				@le0.setText(tr(@toptitle + " Skip...."))
				return
			end
			rssname = "index.rdf"
			if (FileTest.exist?($savedir + dir + "/index.html"))
				File.rename($savedir + dir + "/index.html",$savedir + dir + "/" + rssname)	
			end
			if (FileTest.exist?($savedir + dir + "/.list-snap"))
				File.delete($savedir + dir + "/.list-snap")
			end
		end

		@le0.setText(tr(@toptitle + " cache削除中"))
		begin
		Dir::open($savedir + dir + "/.cache-snap") {|d|
			d.each {|f|
				if ( f =~ /htm/ ) then
					if (FileTest.exist?($savedir + dir + "/.cache-snap/" + f))
						File.delete($savedir + dir + "/.cache-snap/" + f)
					end
				end
			}
		}
		rescue
		end

		htmllist = ""
		savehtmllist = ""
		cachelisturlC = ""
		cachelisturlCI = ""
		begin
			if (FileTest.exist?($savedir + dir + ".xml")) then
				fp2 = File.open($savedir + dir + ".xml","r")
					fp2.each do |line|
						htmllist = htmllist + line.chomp
						if(line =~ /flg="保"/) then
							savehtmllist = savehtmllist + line.chomp
							savelist.push line
						end
						if(line =~ /<list mode=\"C \"/) then
							cachelisturlC  = cachelisturlC  + line.chomp
						end
						if(line =~ /<list mode=\"CI\"/) then
							cachelisturlCI = cachelisturlCI + line.chomp
						end
					end
				fp2.close
			end
		rescue
		end

		@pr0.setProgress(prnum - @step * 0.8 )
		@le0.setText(tr(@toptitle + " RSS解析中"))
		fp = File.open($savedir + dir + "/" + rssname,"r")
			
			doc = REXML::Document.new(fp)
			rssver = doc.version
			
			if $DEBUG then
				print rssver + "\n"
			end

			#0.91 & 2.0
			REXML::XPath.match(doc, "/rss/channel").each do |i|
				@toptitle = i.elements['title'].text
			end
			REXML::XPath.match(doc, "/rss/channel/item").each do |i|
				day = Time.now - timei
				timei = timei + 1
				@yhdate = day.strftime("%y-%m-%d %H:%M")

				@title = i.elements['title'].text
				begin
					if (!i.elements['dc:date'].text.nil?) then
						date  = i.elements['dc:date'].text.sub(/T/," ").sub(/:..\+09:00/,"").sub(/^../,"")
					else
						date = @yhdate
					end
				rescue
					date = @yhdate
				end
				begin
					if (!i.elements['pubDate'].text.nil?) then
						date  = i.elements['pubDate'].text.gsub(/,/,"")
						datenum = ParseDate::parsedate(date)
						date2 = Time::local(*datenum[0..-3])
						date = date2.strftime("%y-%m-%d %H:%M")
					end
				rescue
				end
				tempdate.push date

				url   = i.elements['link'].text
				tempurl = url.split('/')
				urle1 = tempurl.pop
				urle2 = tempurl.pop

				bname = savename_c(url)

				if (bname =~/^index.html$/) then
					bname = urle1 + ".html"
				end

				url2t = $savedir + dir + "/" + bname
				url2 = "file:\/\/" + url2t

				if ( itemcount <= $maxitemnum) then
					if ( htmllist =~ /#{Regexp.quote(escapehtml(url))}/ ) then
						if( savehtmllist =~ /#{Regexp.quote(escapehtml(url))}/ ) then
						else
							if( cachelisturlCI =~ /#{Regexp.quote(escapehtml(url))}/ ) then
								recStr = savestr("CI",@title,"　",date,url2,url)
								templist.push recStr
							elsif( cachelisturlC  =~ /#{Regexp.quote(escapehtml(url))}/ ) then
								recStr = savestr("C ",@title,"　",date,url2,url)
								templist.push recStr
							else
								recStr = savestr("  ",@title,"　",date,url2,url)
								templist.push recStr
							end
						end
						if $w3mbon then
							recw3mb = savew3mb(localc,@title,"　",date,url2,url)
							w3mlist.push recw3mb
						end
					else
						recStr = savestr(localc,@title,"新",date,url2,url)
						templist.push recStr
						if $w3mbon then
							recw3mb = savew3mb(localc,@title,"新",date,url2,url)
							w3mlist.push  recw3mb
						end
						@status = "新"
						if ( FileTest.exist?(url2t)) then
						else
							if( localc =~ /^CI/ ) then
								@le0.setText(tr( @toptitle + " " + @title + " 取得開始"))
								getblog(url, $savedir + dir, 0, " ")
								@le0.setText(tr( @toptitle + " " + @title + " 取得完了"))
							elsif( localc =~ /^C/ ) then
								@le0.setText(tr( @toptitle + " " + @title + " 取得開始"))
								getblog(url, $savedir + dir, 0, "n")
								@le0.setText(tr( @toptitle + " " + @title + " 取得完了"))
							end
							if( localc =~ /^C/ ) then
								if (FileTest.exist?($savedir + dir + "/index.html"))
									File.rename($savedir + dir + "/index.html",$savedir + dir + "/" + bname)	
									if (FileTest.exist?($savedir + dir + "/.cache-snap/index.html"))
										File.delete($savedir + dir + "/.cache-snap/index.html")
									end	
									if (FileTest.exist?($savedir + dir + "/.list-snap"))
										File.delete($savedir + dir + "/.list-snap")
									end
								end
							end
							begin
							Dir::open($savedir + dir + "/.cache-snap") {|d|
								d.each {|f|
									if ( f =~ /htm/ ) then
										if (FileTest.exist?($savedir + dir + "/.cache-snap/" + f))
											File.delete($savedir + dir + "/.cache-snap/" + f)
										end
									end
								}
							}
							rescue
							end
						end
					end
				end
				itemcount = itemcount + 1
			end

			itemcount = 1
			#1.0
			REXML::XPath.match(doc, "/rdf:RDF/channel").each do |i|
				@toptitle = i.elements['title'].text
			end
			REXML::XPath.match(doc, "/rdf:RDF/item").each do |i|
				day = Time.now - timei
				timei = timei + 1
				@yhdate = day.strftime("%y-%m-%d %H:%M")

				@title = i.elements['title'].text
				begin
					if (!i.elements['dc:date'].text.nil?) then
						date  = i.elements['dc:date'].text.sub(/T/," ").sub(/:..\+09:00/,"").sub(/^../,"")				
					else
						date = @yhdate
					end
				rescue
					date = @yhdate
				end
				begin
					if (!i.elements['pubDate'].text.nil?) then
						date  = i.elements['pubDate'].text.gsub(/,/,"")
						datenum = ParseDate::parsedate(date)
						date2 = Time::local(*datenum[0..-3])
						date = date2.strftime("%y-%m-%d %H:%M")
					end
				rescue
				end
				tempdate.push date


				url   = i.elements['link'].text
				tempurl = url.split('/')
				urle1 = tempurl.pop
				urle2 = tempurl.pop

				bname = savename_c(url)


				if (bname =~/^index.html$/) then
					bname = urle1 + ".html"
				end

				url2t = $savedir + dir + "/" + bname
				url2 = "file:\/\/" + url2t


				if ( itemcount <= $maxitemnum) then
					if ( htmllist =~ /#{Regexp.quote(escapehtml(url))}/ ) then
						if( savehtmllist =~ /#{Regexp.quote(escapehtml(url))}/ ) then
						else
							if( cachelisturlCI =~ /#{Regexp.quote(escapehtml(url))}/ ) then
								recStr = savestr("CI",@title,"　",date,url2,url)
								templist.push recStr
							elsif( cachelisturlC  =~ /#{Regexp.quote(escapehtml(url))}/ ) then
								recStr = savestr("C ",@title,"　",date,url2,url)
								templist.push recStr
							else
								recStr = savestr("  ",@title,"　",date,url2,url)
								templist.push recStr
							end
						end
						if $w3mbon then
							recw3mb = savew3mb(localc,@title,"　",date,url2,url)
							w3mlist.push recw3mb
						end
					else
						recStr = savestr(localc,@title,"新",date,url2,url)
						templist.push recStr
						if $w3mbon then
							recw3mb = savew3mb(localc,@title,"新",date,url2,url)
							w3mlist.push recw3mb
						end
						@status = "新"
						if ( FileTest.exist?(url2t)) then
						else
							if( localc =~ /^CI/ ) then
								@le0.setText(tr( @toptitle + " " + @title + " 取得開始"))
								getblog(url, $savedir + dir, 0, " ")
								@le0.setText(tr( @toptitle + " " + @title + " 取得完了"))
							elsif( localc =~ /^C/ ) then
								@le0.setText(tr( @toptitle + " " + @title + " 取得開始"))
								getblog(url, $savedir + dir, 0, " n")
								@le0.setText(tr( @toptitle + " " + @title + " 取得完了"))
							end

							if( localc =~ /^C/ ) then
								if (FileTest.exist?($savedir + dir + "/index.html"))
									File.rename($savedir + dir + "/index.html",$savedir + dir + "/" + bname)	
									if (FileTest.exist?($savedir + dir + "/.cache-snap/index.html"))
										File.delete($savedir + dir + "/.cache-snap/index.html")
									end	
									if (FileTest.exist?($savedir + dir + "/.list-snap"))
										File.delete($savedir + dir + "/.list-snap")
									end
								end
							end
							begin
							Dir::open($savedir + dir + "/.cache-snap") {|d|
								d.each {|f|
									if ( f =~ /htm/ ) then
										if (FileTest.exist?($savedir + dir + "/.cache-snap/" + f))
											File.delete($savedir + dir + "/.cache-snap/" + f)
										end
									end
								}
							}
							rescue
							end
						end
					end
				end
				itemcount = itemcount + 1
			end

			itemcount = 1
			#atom
			REXML::XPath.match(doc, "/feed").each do |i|
				@toptitle = i.elements['title'].text
			end
			REXML::XPath.match(doc, "/feed/entry").each do |i|
				day = Time.now - timei
				timei = timei + 1
				@yhdate = day.strftime("%y-%m-%d %H:%M")

				@title = i.elements['title'].text
				begin
					if (!i.elements['issued'].text.nil?) then
						date  = i.elements['issued'].text.sub(/T/," ").sub(/\+09:00/,"").sub(/^../,"").sub(/:..$/,"").sub(/:..Z$/,"").sub(/Z/,"")				
					else
						date = @yhdate
					end
				rescue
					date = @yhdate
				end
				begin
					if (!i.elements['modified'].text.nil?) then
						date  = i.elements['modified'].text.sub(/T/," ").sub(/\+09:00/,"").sub(/^../,"").sub(/:..$/,"").sub(/:..Z$/,"").sub(/Z/,"")
					end
				rescue
				end
				tempdate.push date

				link = i.get_elements("link").last
				url = link.attributes["href"]

				tempurl = url.split('/')
				urle1 = tempurl.pop
				urle2 = tempurl.pop

				bname = savename_c(url)


				if (bname =~/^index.html$/) then
					bname = urle1 + ".html"
				end

				url2t = $savedir + dir + "/" + bname
				url2 = "file:\/\/" + url2t


				if ( itemcount <= $maxitemnum) then
					if ( htmllist =~ /#{Regexp.quote(escapehtml(url))}/ ) then
						if( savehtmllist =~ /#{Regexp.quote(escapehtml(url))}/ ) then
						else
							if( cachelisturlCI =~ /#{Regexp.quote(escapehtml(url))}/ ) then
								recStr = savestr("CI",@title,"　",date,url2,url)
								templist.push recStr
							elsif( cachelisturlC  =~ /#{Regexp.quote(escapehtml(url))}/ ) then
								recStr = savestr("C ",@title,"　",date,url2,url)
								templist.push recStr
							else
								recStr = savestr("  ",@title,"　",date,url2,url)
								templist.push recStr
							end
						end
						if $w3mbon then
							recw3mb = savew3mb(localc,@title,"　",date,url2,url)
							w3mlist.push recw3mb
						end
					else
						recStr = savestr(localc,@title,"新",date,url2,url)
						templist.push recStr
						if $w3mbon then
							recw3mb = savew3mb(localc,@title,"新",date,url2,url)
							w3mlist.push recw3mb
						end
						@status = "新"
						if ( FileTest.exist?(url2t)) then
						else
							if( localc =~ /^CI/ ) then
								@le0.setText(tr( @toptitle + " " + @title + " 取得開始"))
								getblog(url, $savedir + dir, 0, " ")
								@le0.setText(tr( @toptitle + " " + @title + " 取得完了"))
							elsif( localc =~ /^C/ ) then
								@le0.setText(tr( @toptitle + " " + @title + " 取得開始"))
								getblog(url, $savedir + dir, 0, " n")
								@le0.setText(tr( @toptitle + " " + @title + " 取得完了"))
							end

							if( localc =~ /^C/ ) then
								if (FileTest.exist?($savedir + dir + "/index.html"))
									File.rename($savedir + dir + "/index.html",$savedir + dir + "/" + bname)	
									if (FileTest.exist?($savedir + dir + "/.cache-snap/index.html"))
										File.delete($savedir + dir + "/.cache-snap/index.html")
									end	
									if (FileTest.exist?($savedir + dir + "/.list-snap"))
										File.delete($savedir + dir + "/.list-snap")
									end
								end
							end
							begin
							Dir::open($savedir + dir + "/.cache-snap") {|d|
								d.each {|f|
									if ( f =~ /htm/ ) then
										if (FileTest.exist?($savedir + dir + "/.cache-snap/" + f))
											File.delete($savedir + dir + "/.cache-snap/" + f)
										end
									end
								}
							}
							rescue
							end
						end
					end
				end
				itemcount = itemcount + 1
			end
		fp.close()

		@newdate = tempdate[0]	
		recStr = savestr(localc,@toptitle,@status,@newdate,rssurl," ")
		@temptop.push recStr


		@pr0.setProgress(prnum - @step * 0.5 )	
		if $w3mbon then
			fp2 = File.open($savedir + "w3m" + dir + ".html","w")
	   			fp2.puts("<html><head><title>zRSSReader Bookmarks</title></head>")
				fp2.puts("<body>")
 				fp2.puts("<h2>" + @toptitle + "</h2>")
				fp2.puts("<ul>")
				w3mlist.each do |line|
					fp2.puts(line)
				end 
				fp2.puts("</ul>")
				fp2.puts("</body>")
				fp2.puts("</head>")
			fp2.close
		end

		@pr0.setProgress(prnum - @step * 0.4 )	

		htmllist = ""
		fp2 = File.open($savedir + dir + ".xml","w")
			fp2.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp2.puts("<zRSSReader>")
			templist.each do |line|
				fp2.puts(line)
				htmllist = htmllist + line.chomp
			end
			savelist.each do |line|
				fp2.puts(line)
				htmllist = htmllist + line.chomp
			end
   			fp2.puts("</zRSSReader>")
		fp2.close

		@pr0.setProgress(prnum - @step * 0.3 )	
		Dir::open($savedir + dir) {|d|
			d.each {|f|
				if ( f =~ /htm/ )
					if ( htmllist =~ /#{Regexp.quote(f.sub(/\.html/,""))}/ )
					else
						if (FileTest.exist?($savedir + dir + "/" + f))
							File.delete($savedir + dir + "/" + f)
							print "del " + f + "\n"
						end
					end
				end
			}
		}

		@pr0.setProgress(prnum - @step * 0.2 )
		if (FileTest.exist?($savedir + dir + "/" + rssname))
			File.delete($savedir + dir + "/" + rssname)
		end

		if (FileTest.exist?($savedir + dir + "/.cache-snap/" + rssname + ".html"))
			File.delete($savedir + dir + "/.cache-snap/" + rssname + ".html")	
		end

		if (FileTest.exist?($savedir + dir + "/.cache-snap/" + rssname))
			File.delete($savedir + dir + "/.cache-snap/" + rssname)	
		end

		@pr0.setProgress(prnum - @step * 0.1 )
		@le0.setText(tr(@toptitle + " 取得終了"))

		templist = []
		savelist = []
		tempurl = []
		tempdate = []
		datenum = []
		w3mlist = []

		GC.start
	end

	def savename_c(url)
		uri = URI.parse(url)
		path = uri.path
		bname = path =~ /\/$/ ? "/" : File.basename(path)
		bname = "index.html" if bname.empty? || bname == "/"
		bname << ".html" if bname !~ /\.s?html?$/i

		base, ext = bname =~ /^(.*)(\.[^.]*)$/ ? [$1, $2] : [bname, ""]

		unless /^index\.s?html?$/i =~ bname
			base = change_name_by_salt2(base, uri.host, File.dirname(uri.path))
		end

		bname =  base + ext
		bname
	end

	def change_name_by_salt2(base, host, dir)

		pths = dir.split(/\//)
		if pths.empty?
			pths = ["_root"]
		end

		if /[a-zA-Z].*[a-zA-Z]/ =~ base
		elsif /(?:www\.)([a-zA-Z]+)/ =~ host
			base = (pths[-1..-1] + [base]).join('-')
		else
			base = (pths[-1..-1] + [base]).join('-')
		end
		base
	end

	def getblog(target, savedir, depth, option)
	  $KCODE = "s"
	  getopts("DijEUuxhtvVgwaAes",
		  "c:", "d:", "p:", "m:", "n:", "f:.httpsnapshotrc", "y:")

	  if $OPT_A
	    $OPT_a = true
	  end

	  if (option =~ /g/) != nil then
		$OPT_g = true
  	  else
		$OPT_g = false
	  end

	  if (option =~ /n/) != nil then
		$OPT_y = "/home/zaurus/Settings/zRSSReader_filter_ni.txt"	
  	  else
		$OPT_y = "/home/zaurus/Settings/zRSSReader_filter.txt"	
	  end

	  $config = HttpSnapSession.read_config($OPT_f) if $OPT_f


	  if $proxyon then
 	 	 proxy_s = $proxyurl
	  else
 		 proxy_s = "" # ENV["http_proxy"] #"" # $OPT_p || $config["proxy"] || ENV["HTTPSNAPSHOT_PROXY"] || ENV["http_proxy"]
	  end

	  if proxy_s && !proxy_s.empty?
	    proxy_s = proxy_s.sub(/^(:?http:\/\/)?/, 'http://')
	    proxy_s.concat ":8080" if proxy_s !~ /:\d+$/
	    proxy = URI.parse(proxy_s)
	    proxy, proxy_port = proxy.host, proxy.port
	  end

	  session = HttpSnapSession.new(target, savedir, depth)
	  session.cache_dir = $OPT_c if $OPT_c
	  session.verbose =  $DEBUG # $OPT_v || !$OPT_V # 
	  session.depth_first = $OPT_D
	#  session.false = !$OPT_U || $OPT_u
	  session.proxy, session.proxy_port = proxy, proxy_port

	  if (option =~ /U/) != nil then
		$OPT_U = true
		session.read_local = false 
  	  else
		$OPT_U = false
		session.read_local = true  
	  end

	  if $OPT_j
	    session.href_same_host = false
	  end
	  if $OPT_i
	    session.src_same_host = false
	  end
	  session.only_get = true if $OPT_g || $OPT_a
	  session.output_format = "text" if $OPT_a
	  session.output_rs = "\r\n" if $OPT_A
	  session.user_agent = "Mozilla/5.0 (Windows; U; Windows NT 5.1; ja-JP; rv:1.0.2) Gecko/20030208 Netscape/7.02"

	  # Add filters
	  unless $OPT_x
	    session.add_href_filter do |snap, psnap|
	      snap.upper_dir?(session.root) || snap.path =~ /\bcgi-bin\b/
	    end
	  end

	  if $OPT_t || $OPT_a
	    session.add_href_filter do |snap, psnap| snap.guess_text?; end
	    session.add_src_filter do |snap, psnap| snap.guess_text?; end
	  end
	
	  if $OPT_n
	    min = $OPT_n.to_i
	    session.add_pre_filter do |r|
	      ctype, len = r["content-type"], r["content-length"].to_i
	      not /\bimage\b/ =~ ctype && len < min
	    end
	  end

	  if $OPT_m
	    max = $OPT_m.to_i
 	   session.add_pre_filter do |r|
 	     ctype, len = r["content-type"], r["content-length"].to_i
 	     not /\bimage\b/ =~ ctype && len > max
 	   end
 	 end

#	$OPT_y = "/home/zaurus/Settings/zRSSReader_filter.txt"	
	  if $OPT_y
	    src_allows = []
	    src_denys = []
	    href_allows = []
	    href_denys = []
	    open($OPT_y) do |fi|
	      mode = igc = nil
	      fi.each do |line|
	        line.chomp!
	        line.strip!
	        line.sub!(/#.*$/, '')
	
	        case line
	        when /^:(.*)/
	          inst = $1.split(/,\s*/)
	          inst.each do |x|
	            case x
	            when "src-allow"; mode = :srcallow
	            when "src-deny"; mode = :srcdeny
	            when "href-allow"; mode = :hrefallow
	            when "href-deny"; mode = :hrefdeny
	            when "ignorecase"; igc = true
	            when "casesensitive"; igc = false
	            else
	              raise "unknown option (#$OPT_y): '#{x}'"
	            end
	          end
	        when /\S/
	          case mode
	          when :srcallow
	            r = igc ? Regexp.new(line, Regexp::IGNORECASE) : Regexp.new(line)
	            src_allows.push r
	          when :srcdeny
	            r = igc ? Regexp.new(line, Regexp::IGNORECASE) : Regexp.new(line)
	            src_denys.push r
	          when :hrefallow
	            r = igc ? Regexp.new(line, Regexp::IGNORECASE) : Regexp.new(line)
	            href_allows.push r
	          when :hrefdeny
	            r = igc ? Regexp.new(line, Regexp::IGNORECASE) : Regexp.new(line)
	            href_denys.push r
	          else
	            raise "unkonw paremteter '#{mode}' in config #$OPT_y"
	          end
	        end
	      end
	    end
	
	    session.add_src_filter do |snap, psnap|
	      name = snap.path
	      src_allows.any?{|r| r =~ name} and !src_denys.any?{|r| r =~ name}
	    end
	
	    session.add_href_filter do |snap, psnap|
	      name = snap.path
	      href_allows.any?{|r| r =~ name} and !href_denys.any?{|r| r =~ name}
	    end
	  end
	
	  # Session
	  if $OPT_E
	    session.table_file_clear
	    session.cache_clear
	  end

	session.href_same_host = true
	session.src_same_host = false

	  if $OPT_w
	    session.build_table
	    session.rewrite
	  else
	    session.start
	  end

	  # post session
	  if $OPT_e
	    session.cache_clear
	  end
	  if session.first && !session.ostdio
		if $DEBUG then
		    puts "First File: #{session.start_file}"
		end
	  end

	  GC.start
	end

	def resizeEvent ev
		titlewidth = QPEApplication.desktop.width - 20 - 30 -90 -40
		@bloglist.setColumnWidth(1,titlewidth)
	end

end

class BerryRSSDescViewWidget < BerryBaseWidget
	def initialize(parent)
		super(parent)
		
		@parent = parent

		@contentPain = BerryRSSDescViewMultiLineView.new(self)
		#@bloglist = QListView.new(self)
		@contentPain.setAllColumnsShowFocus(true)
		titlewidth = QPEApplication.desktop.width - 20 - 30 -90 -49
		@contentPain.addColumn(tr("S"),20)
		@contentPain.addColumn(tr("Title"),titlewidth)
		@contentPain.addColumn(tr("新着"),30)
		@contentPain.addColumn(tr("date"),90)
		@contentPain.setRootIsDecorated(false)
		@contentPain.setMultiSelection(false)
		@contentPain.setFont QFont.new($fontname,$fontsize,QFont::Bold)

		@contentPain.setColumnAlignment(2,AlignHCenter)

		@Menu1 = QPopupMenu.new()
		@Menu1.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@m11 = @Menu1.insertItem(tr("キャッシュ設定ONOFF"), self, "cacherss", Key_C)
		@m12 = @Menu1.insertItem(tr("すべてのキャッシュ設定ONOFF"), self, "cacheallrss")
		@m13 = @Menu1.insertItem(tr("この記事を保持する"), self, "saverss", Key_S)
		@m14 = @Menu1.insertItem(tr("すべての記事を保持する"), self, "allsaverss")


		@menuBar = QPEMenuBar.new(self)
		@menuBar.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@menuBar.insertItem(tr("RSS設定"), @Menu1)

		@le0 = QLabel.new("",self)
		@le0.setIndent(5)
		@le0.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		
		@widgetLayout = QVBoxLayout.new(self)
		@widgetLayout.addWidget(@menuBar)
		@widgetLayout.addWidget(@le0)
		@widgetLayout.addWidget(@contentPain)

		connect(@contentPain,QSIGNAL("returnPressed(QListViewItem *)"), self, 'openbro')
		connect(@contentPain,QSIGNAL("doubleClicked(QListViewItem *)"), self, 'openbro')		

	end

	def close()
		@parent.changeScreen(Berry::RSS_LIST_WIDGET_ID)
		return false
	end

	def setsortlist
		@contentPain.setSorting(3,false)
		item = @contentPain.firstChild()
		@contentPain.setCurrentItem(item)
		@contentPain.setFocus()
	end

	def screenInit(rssItem = nil,rsstitle=nil,returnScreenId=Berry::RSS_LIST_WIDGET_ID)
		savelist = []
		savelist= rssItem.split('/')
		@contentPain.clear()
		@sedlist = savelist[2].gsub(/\./,"") + "_" + savelist[3].gsub(/\./,"") + "_" + savelist[-1].gsub(/\./,"").sub(/\?/,"") 
		readbloglist($savedir + @sedlist + ".xml")
		setsortlist
		@le0.setText(tr(rsstitle))

		titlewidth = QPEApplication.desktop.width - 20 - 30 -90 -40
		@contentPain.setColumnWidth(1,titlewidth)
	end

	def openbro(item)
		list   = item.text(4).utf8().to_str()
		onlist = item.text(5).utf8().to_str()
		localc = item.text(0).utf8().to_str()
		chlist = list.sub(/file:\/\//,'')
		if( localc =~ /^C/ ) then
			if ( FileTest.exist?(chlist)) then
				system("qcop QPE/System 'execute(QString,QString)'  " + $browser + "  '" + list + "'")
			elsif ( FileTest.exist?(chlist + "\.html")) then
				system("qcop QPE/System 'execute(QString,QString)'  " + $browser + "  '" + list + "\.html'")
			else
				begin
				listtmp = list.split('/')
				temp = listtmp.pop.split('-')
				list = ""
				listtmp.each do |line|
					list1 = list
					list = list1 + line + "\/"
				end
				system("qcop QPE/System 'execute(QString,QString)' " + $browser + "  '" + list + "\/" + temp[1] + "'")
				rescue
				end
			end
		else
			system("qcop QPE/System 'execute(QString,QString)' " + $browser + " '" + onlist + "'")
		end	
	end

	def readbloglist(filePath)
	        fp = File.new(filePath, "r")
		        handler = ZcDataParser.new()
		        source = QXmlInputSource.new()
		        source.setData(tr(fp.read))
		        reader = QXmlSimpleReader.new()
		        reader.setContentHandler(handler)
		        reader.parse(source)
	        fp.close()
	        recordList = handler.recordList()
	        if (recordList.nil?)
			return false
	        end

	        for i in 0..(recordList.length() - 1)
			y1 = recordList[i][0]
			y2 = recordList[i][1]
			y3 = recordList[i][2]
			y4 = recordList[i][3]
			y5 = recordList[i][4]
			y6 = recordList[i][5]
			QListViewItem.new(@contentPain,tr(y1),tr(y2),tr(y3),tr(y4),tr(y5),tr(y6))
	        end	
	end

	def cacherss
		begin
			item = @contentPain.selectedItem()
			rec = row(item)
			if (rec[0] =~ /CI/ ) then
				item.setText(0,"C ")
			elsif (rec[0] =~ /C/ ) then
				item.setText(0,"  ")
			else 
				item.setText(0,"CI")
			end

			item = @contentPain.firstChild()
			fp = File.open($savedir + @sedlist + ".xml","w")
			fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp.puts("<zRSSReader>")
			while(!item.nil?)
				rec = row(item)
				recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])
				fp.puts	(recStr)
				item = item.nextSibling()
			end
			fp.close
		rescue
		end
	end


	def cacheallrss
		begin
			item = @contentPain.firstChild()
			rec = row(item)
			if (rec[0] =~ /CI/ ) then
				cachec = "C "
			elsif (rec[0] =~ /C/ ) then
				cachec = "  "
			else 
				cachec = "CI"
			end

			item = @contentPain.firstChild()
			while(!item.nil?)
				item.setText(0,cachec)
				rec = row(item)
				item = item.nextSibling()
			end

			item = @contentPain.firstChild()
			fp = File.open($savedir + @sedlist + ".xml","w")
			fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp.puts("<zRSSReader>")
			while(!item.nil?)
				rec = row(item)
				recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])
				fp.puts	(recStr)
				item = item.nextSibling()
			end
			fp.close
		rescue
		end
	end

	def saverss
		begin
			item = @contentPain.selectedItem()
			rec = row(item)
			if (rec[2] =~ /保/ ) then
				item.setText(2,tr("　"))
			else 
				item.setText(2,tr("保"))
			end

			item = @contentPain.firstChild()
			fp = File.open($savedir + @sedlist + ".xml","w")
			fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp.puts("<zRSSReader>")
			while(!item.nil?)
				rec = row(item)
				recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])
				fp.puts	(recStr)
				item = item.nextSibling()
			end
			fp.close
		rescue
		end
	end

	def allsaverss
		begin
			item = @contentPain.selectedItem()
			rec = row(item)
			if (rec[2] =~ /保/ ) then
				cachec = tr("　")
			else 
				cachec = tr("保")
			end

			item = @contentPain.firstChild()
			fp = File.open($savedir + @sedlist + ".xml","w")
			fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp.puts("<zRSSReader>")
			while(!item.nil?)
				item.setText(2,cachec)
				rec = row(item)
				recStr = savestr(rec[0],rec[1],rec[2],rec[3],rec[4],rec[5])
				fp.puts	(recStr)
				item = item.nextSibling()
			end
			fp.close
		rescue
		end
	end

	def savestr(str0,str1,str2,str3,str4,str5)
		recStr = "\t<list mode=\""
		recStr += str0
		recStr += "\" title=\""
		recStr += escapehtml(str1).chomp.gsub(/\r/m,"").gsub(/\n/m,"")
		recStr += "\" flg=\""
		recStr += str2
		recStr += "\" date=\""
		recStr += str3
		recStr += "\" url=\""
		recStr += escapehtml(str4)
		recStr += "\" url2=\""
		recStr += escapehtml(str5)
		recStr += "\" />\n"
		return recStr
	end

	def escapehtml(str)
		str = str.gsub(/\&/,"\&amp\;").gsub(/"/m,"\&quot\;").gsub(/</,"\&lt\;").gsub(/>/m,"\&gt\;").gsub(/'/m,"\&apos\;")
		return str
	end

	def row(item)
		rec = Array.new(6)
		rec[0] = item.text(0).utf8().to_str()
		rec[1] = item.text(1).utf8().to_str()
		rec[2] = item.text(2).utf8().to_str()
		rec[3] = item.text(3).utf8().to_str()
		rec[4] = item.text(4).utf8().to_str()
		rec[5] = item.text(5).utf8().to_str()
		return rec
	end

end

class BerryRSSDescViewMultiLineView < QListView
	def initialize(parent)
		super(parent)
		@parent = parent
	end
end


class BloglistDialog < QDialog
	def initialize(parnet, name= '')
		super(parnet, name,true)

		setCaption(tr("RSS追加"))
		@result = false

		bg2 = QVButtonGroup.new(self)
		bg2.setGeometry(0,20,QPEApplication.desktop.width - 100,80)
		bg2.setFont QFont.new($fontname,$fontsize,QFont::Bold)

		@le1 = QLineEdit.new("",bg2)
		@pb1 = QPushButton.new(tr("追加"),bg2)

		Qte::connect(@pb1,QSIGNAL("clicked()"), self, "addrssgo()")
	end

	def addrssgo()
		fp = File.open($homedir + "zRSSRtop.xml","a")
			recStr = savestr("  ","タイトルは自動取得します","　","　",@le1.text.to_str,"　")
			fp.puts(recStr)
		fp.close
		@result = true
		close()
	end

	def result2()
		return @result
	end
	def savestr(str0,str1,str2,str3,str4,str5)
		recStr = "\t<list mode=\""
		recStr += str0
		recStr += "\" title=\""
		recStr += escapehtml(str1).chomp.gsub(/\r/m,"").gsub(/\n/m,"")
		recStr += "\" flg=\""
		recStr += str2
		recStr += "\" date=\""
		recStr += str3
		recStr += "\" url=\""
		recStr += escapehtml(str4)
		recStr += "\" url2=\""
		recStr += escapehtml(str5)
		recStr += "\" />\n"
		return recStr
	end

	def escapehtml(str)
		str = str.gsub(/\&/,"\&amp\;").gsub(/"/m,"\&quot\;").gsub(/</,"\&lt\;").gsub(/>/m,"\&gt\;").gsub(/'/m,"\&apos\;")
		return str
	end

end

class RssAdDialog < QDialog
	def initialize(parnet, name= '')
		super(parnet, name,true)

		setCaption(tr("RSS Autodiscovery"))
		@result = false

		bg5 = QVButtonGroup.new(self)
		bg5.setGeometry(0,20,QPEApplication.desktop.width - 100,200)
		bg5.setFont QFont.new($fontname,$fontsize,QFont::Bold)

		@le2 = QLabel.new(tr("RSSのURLを抽出します。Blog TOPページのURLを入力して下さい。"),bg5)
		@le1 = QLineEdit.new("",bg5)
		@pb1 = QPushButton.new(tr("抽出開始"),bg5)

		@lb1 = QListBox.new(bg5)
		@le0 = QLabel.new("",bg5)


		@lb1.setFont QFont.new($fontname,$fontsize,QFont::Bold)
		@le0.setFont QFont.new($fontname,$fontsize,QFont::Bold)

		Qte::connect(@pb1,QSIGNAL("clicked()"), self, "rssAdgo()")
		Qte::connect(@lb1,QSIGNAL("returnPressed(QListBoxItem *)"), self, 'addrssgo')
	end

	def rssAdgo()

	    @t = Thread.start do
		Thread.pass

		@le0.setText(tr("取得中"))
		templine = []
		fp = File.open($homedir + "rssAd.htm","w")
			begin
			timeout(60) {
				open(@le1.text.to_str) do |f|
					f.each_line() do |line|
						fp.puts(line)
					end
				end
			}
			rescue
			rescue TimeoutError
				@le0.setText(tr("timeout 60sec"))
			end
		fp.close
		@le0.setText(tr("解析中"))
		@lb1.clear
		begin
		timeout(30) {
			fp = File.open($homedir + "rssAd.htm")
			while line = fp.gets
				begin
					templine =  Uconv.euctou8(line).scan(/alternate\"type\=\"application\/(\S+)\+(\S+)\"title\=\"(\S+)\"href\=\"([\S\d\-\_\/\.\:]+)"/)
					templine.each do |line|
							@lb1.insertItem(line[3].to_s)
					end
				rescue
				end
	
				begin
					templine =  Uconv.euctou8(line).scan(/<link title\=\"(\S+)\"href\=\"([\S\d\-\_\/\.\:]+)\"type\=\"application\/rss\+xml\"rel\=\"alternate\"\/>/)
					templine.each do |line|
							@lb1.insertItem(line[1].to_s)
					end
 				rescue
				end
	
				begin
					templine2 =  Uconv.sjistou8(line)
					if templine2 =~ /alternate/ then
						if templine2 =~ /application/ then
							if templine2 =~ /rss|atom/
								templine2t = templine2.scan(/href\=\"([\S\d\-\_\/\.\:]+)\"/) 
					#			@lb1.insertItem(templine2t[0].to_s)
								templine2t.each do |line|
									if line[0] =~ /rss|atom|rdf|xml/ then
										@lb1.insertItem(line[0].to_s)
									end
								end
							end
						end
					end
				rescue
				end
	
			end
		}

		rescue
		rescue TimeoutError
		end

		@le0.setText(tr("Enterで登録"))
		end
	end

	def addrssgo(item)
		title  = item.text.utf8().to_str()
		fp = File.open($homedir + "zRSSRtop.xml","a")
			recStr = savestr("  ","タイトルは自動取得します","　","　",title,"　")
			fp.puts(recStr)
		fp.close
		@result = true
		close()
	end

	def result2()
		return @result
	end

	def savestr(str0,str1,str2,str3,str4,str5)
		recStr = "\t<list mode=\""
		recStr += str0
		recStr += "\" title=\""
		recStr += escapehtml(str1).chomp.gsub(/\r/m,"").gsub(/\n/m,"")
		recStr += "\" flg=\""
		recStr += str2
		recStr += "\" date=\""
		recStr += str3
		recStr += "\" url=\""
		recStr += escapehtml(str4)
		recStr += "\" url2=\""
		recStr += escapehtml(str5)
		recStr += "\" />\n"
		return recStr
	end

	def escapehtml(str)
		str = str.gsub(/\&/,"\&amp\;").gsub(/"/m,"\&quot\;").gsub(/</,"\&lt\;").gsub(/>/m,"\&gt\;").gsub(/'/m,"\&apos\;")
		return str
	end

end

class BlogeditDialog < QDialog
	def initialize(parnet = nil , name = '')
		super(parnet, name, true)

		setCaption(tr("RSS修正"))
		@result = false
		bg4 = QVButtonGroup.new(self)
		bg4.setGeometry(0,20,QPEApplication.desktop.width - 100,80)
		bg4.setFont QFont.new($fontname,$fontsize,QFont::Bold)

		@le4 = QLineEdit.new($editrss,bg4)
		@pb4 = QPushButton.new(tr("修正"),bg4)

		connect(@pb4,QSIGNAL("clicked()"), self, 'editrssgo')
	end

	def editrssgo
		$editrss = @le4.text.to_str
		@result = true
		close()
	end

	def result2()
		return @result
	end
end

class BlogSettingDialog < QDialog
	def initialize(parnet = nil, name = '')
		super(parnet, name)
		@result = false

		setCaption(tr("zRSSReader オプション設定"))
		@result = false
		bg3 = QVButtonGroup.new(self)
		bg3.setGeometry(0,20,QPEApplication.desktop.width - 50,QPEApplication.desktop.height - 110)
		bg3.setFont QFont.new($fontname,$fontsize,QFont::Bold)

		@sl31 = QLabel.new(tr("保存Path"),bg3)
		@le31 = QLineEdit.new($savedir,bg3)
		@sl32 = QLabel.new(tr("最大記事数"),bg3)
		@le32 = QLineEdit.new($maxitemnum.to_s,bg3)
		@sl33 = QLabel.new(tr("Font Size"),bg3)
		@cb31 = QComboBox.new(bg3)

		@cb31.insertItem("12")
		@cb31.insertItem("14")
		@cb31.insertItem("16")
		@cb31.insertItem("18")
		@cb31.setCurrentItem($fontsize_item)

		@cc34 = QCheckBox.new(tr("Proxyサーバを使用する(入力例:http://proxy.xxx.co.jp:8080/)"),bg3)
		@cc34.setChecked($proxyon)
		@le34 = QLineEdit.new($proxyurl,bg3)

		@sl35 = QLabel.new(tr("ブラウザ"),bg3)
		@cb36 = QComboBox.new(bg3)

		@cb36.insertItem("Netfront")
		@cb36.insertItem("Opera")
		@cb36.setCurrentItem($browser_item)

		@cc37 = QCheckBox.new(tr("w3m用HTMLファイルを出力する"),bg3)
		@cc37.setChecked($w3mbon)

		@pb31 = QPushButton.new(tr("保存"),bg3)

		connect(@pb31,QSIGNAL("clicked()"),self,'savesetfile')

	end

	def savesetfile
		@savedir_temp = @le31.text.to_str
		@maxitemnum_temp = @le32.text.to_str
		$fontsize_item = @cb31.currentItem.to_i

		case @cb31.currentItem.to_i
		when 0
			@fontsize_temp = "12"
		when 1
			@fontsize_temp = "14"
		when 2
			@fontsize_temp = "16"
		when 3
			@fontsize_temp = "18"
		end

		$savedir = @savedir_temp
		$maxitemnum = @maxitemnum_temp
		$fontsize = @fontsize_temp.to_i

		@proxyon_temp = @cc34.isChecked.to_s
		@proxyurl_temp = @le34.text.to_str

		$proxyon = @cc34.isChecked
		$proxyurl = @proxyurl_temp

		case @cb36.currentItem.to_i
		when 0
			@browser_temp = "netfront3"
		when 1
			@browser_temp = "opera"
		end
		$browser = @browser_temp
		$browser_item = @cb36.currentItem.to_i

		@w3mbon_temp = @cc37.isChecked.to_s
		$w3mbon = @cc37.isChecked

		fp = File.new($setfile,"w")
			fp.puts("<?xml version='1.0' encoding='UTF-8' ?>")
			fp.puts("<config>")

			str = "\t<savedir>"
			if(!@savedir_temp.nil?)
				str += @savedir_temp
			end
			str += "</savedir>\n"
			fp.puts(str)

			str = "\t<maxitem>"
			if(!@maxitemnum_temp.nil?)
				str += @maxitemnum_temp
			end
			str += "</maxitem>\n"
			fp.puts(str)

			str = "\t<fontsize>"
			if(!@fontsize_temp.nil?)
				str += @fontsize_temp
			end
			str += "</fontsize>\n"
			fp.puts(str)

			str = "\t<fontsize_item>"
			if(!@cb31.currentItem.to_s.nil?)
				str += @cb31.currentItem.to_s
			end
			str += "</fontsize_item>\n"
			fp.puts(str)

			str = "\t<proxyon>"
			if(!@proxyon_temp.nil?)
				str += @proxyon_temp
			end
			str += "</proxyon>\n"
			fp.puts(str)

			str = "\t<proxyurl>"
			if(!@proxyurl_temp.nil?)
				str += @proxyurl_temp
			end
			str += "</proxyurl>\n"
			fp.puts(str)

			str = "\t<browser>"
			if(!@browser_temp.nil?)
				str += @browser_temp
			end
			str += "</browser>\n"
			fp.puts(str)

			str = "\t<browser_item>"
			if(!@cb36.currentItem.to_s.nil?)
				str += @cb36.currentItem.to_s
			end
			str += "</browser_item>\n"
			fp.puts(str)

			str = "\t<w3m_bookmark>"
			if(!@w3mbon_temp.nil?)
				str += @w3mbon_temp
			end
			str += "</w3m_bookmark>\n"
			fp.puts(str)

			fp.puts("</config>")
		fp.close()
		@result = true
		close()
	end

	def result2()
		return @result
	end
end

class OpmlDialog < QDialog
	def initialize(parnet = nil , name = '')
		super(parnet, name, true)

		setCaption(tr("OPML"))
		@result = false
		bg4 = QVButtonGroup.new(self)
		bg4.setGeometry(0,20,QPEApplication.desktop.width - 100,80)
		bg4.setFont QFont.new($fontname,$fontsize,QFont::Bold)

		@le4 = QLineEdit.new($savedir + "inport.xml",bg4)
		@pb4 = QPushButton.new(tr("OPML読み込み"),bg4)

		connect(@pb4,QSIGNAL("clicked()"), self, 'setopmlgo')
	end

	def setopmlgo
		$opmlfile = @le4.text.to_str
		@result = true
		close()
	end

	def result2()
		return @result
	end
end

class ZcDataParser < QXmlRubyHandler

	def startDocument()
		@state = 0
		@recordList = nil
		return true
	end

	def startElement(namespaceURI, qstr1, qName, attributes)
		if (qName.to_str == "zRSSReader" && @state == 0)
			@state = 1
			@recordList = Array.new()
		elsif (qName.to_str == "list" && @state == 1)
			if (attributes.length() > 0)
				rec = Array.new(6)
				for i in 0..(attributes.length() - 1)
					if (attributes.qName(i).to_str == "mode")
						rec[0] = attributes.value(i).utf8().to_str()
					elsif (attributes.qName(i).to_str == "title")
						rec[1] = attributes.value(i).utf8().to_str()
					elsif (attributes.qName(i).to_str == "flg")
						rec[2] = attributes.value(i).utf8().to_str()
					elsif (attributes.qName(i).to_str == "date")
						rec[3] = attributes.value(i).utf8().to_str()
					elsif (attributes.qName(i).to_str == "url")
						rec[4] = attributes.value(i).utf8().to_str()
					elsif (attributes.qName(i).to_str == "url2")
						rec[5] = attributes.value(i).utf8().to_str()
					end
				end
				@recordList.push(rec)
			end
		else
			# NA
		end
		return true
	end

	def endElement(qstr1, qstr2, qstr3)
		return true
	end

	def endDocument()
		return true
	end

	def recordList()
		return @recordList
	end
end

class ZcOpmlParser < QXmlRubyHandler

	def startDocument()
		@state = 0
		@recordList = nil
		return true
	end

	def startElement(namespaceURI, qstr1, qName, attributes)
		if (qName.to_str == "opml" && @state == 0)
			@state = 1
			@recordList = Array.new()
		elsif (qName.to_str == "outline" && @state == 1)
			if (attributes.length() > 0)
				rec = Array.new(2)
				for i in 0..(attributes.length() - 1)
					if (attributes.qName(i).to_str == "title")
						rec[0] = attributes.value(i).utf8().to_str()
					elsif (attributes.qName(i).to_str == "xmlUrl")
						rec[1] = attributes.value(i).utf8().to_str()
					end
				end
				@recordList.push(rec)
			end
		else
			# NA
		end
		return true
	end

	def endElement(qstr1, qstr2, qstr3)
		return true
	end

	def endDocument()
		return true
	end

	def recordList()
		return @recordList
	end
end

app = QPEApplication.new([$0]+ARGV)
app.setDefaultCodec(QTextCodec::codecForName('utf8'))
berry = Berry.new()
app.showMainWidget(berry)
app.exec()